@@ -1,19 +1,30 @@
-# $Id: Ping.pm,v 1.8 2002/10/10 15:58:22 rcaputo Exp $
+# $Id: Ping.pm,v 1.19 2004/05/31 17:24:07 rcaputo Exp $
 # License and documentation are after __END__.
 
 package POE::Component::Client::Ping;
 
-use Exporter;
-@ISA = qw(Exporter);
-@EXPORT_OK = qw(REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS
-		RES_ADDRESS RES_ROUNDTRIP RES_TIME);
-%EXPORT_TAGS = ('const' => [ qw(REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS
-				RES_ADDRESS RES_ROUNDTRIP RES_TIME) ]
-		);
+use warnings;
 use strict;
 
+use Exporter;
+use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
+
+@ISA = qw(Exporter);
+@EXPORT_OK = qw(
+  REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS RES_ADDRESS
+  RES_ROUNDTRIP RES_TIME
+);
+%EXPORT_TAGS = (
+  const => [
+    qw(
+      REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS RES_ADDRESS
+      RES_ROUNDTRIP RES_TIME
+    )
+  ]
+);
+
 use vars qw($VERSION);
-$VERSION = '0.98';
+$VERSION = '1.04';
 
 use Carp qw(croak);
 use Symbol qw(gensym);
@@ -22,8 +33,9 @@ use Time::HiRes qw(time);
 
 use POE::Session;
 
-sub DEBUG        () { 0 }; # Enable more information.
-sub DEBUG_SOCKET () { 0 }; # Watch the socket open and close.
+sub DEBUG        () { 0 } # Enable more information.
+sub DEBUG_SOCKET () { 0 } # Watch the socket open and close.
+sub DEBUG_PBS    () { 0 } # Watch ping_by_seq management.
 
 # Spawn a new PoCo::Client::Ping session.  This basically is a
 # constructor, but it isn't named "new" because it doesn't create a
@@ -36,10 +48,10 @@ sub spawn {
   my %params = @_;
 
   croak "$type requires root privilege"
-    if $> and ($^O ne 'VMS') and not defined $params{Socket};
+    if $> and ($^O ne "VMS") and not defined $params{Socket};
 
   my $alias = delete $params{Alias};
-  $alias = 'pinger' unless defined $alias and length $alias;
+  $alias = "pinger" unless defined $alias and length $alias;
 
   my $timeout = delete $params{Timeout};
   $timeout = 1 unless defined $timeout and $timeout >= 0;
@@ -87,7 +99,7 @@ sub RES_TIME          () { 2 };
 # "Static" variables which will be shared across multiple instances.
 
 my $pid = $$ & 0xFFFF;
-my $seq = 0;
+my $master_seq = 0;
 
 # Start the pinger session.  Record running stats, and create the
 # socket which will be used to ping.
@@ -149,17 +161,17 @@ sub poco_ping_ping {
 
   # Find an unused sequence number.
   while (1) {
-    $seq = ($seq + 1) & 0xFFFF;
-    last unless exists $heap->{ping_by_seq}->{$seq};
+    $master_seq = ($master_seq + 1) & 0xFFFF;
+    last unless exists $heap->{ping_by_seq}->{$master_seq};
   }
 
   my $checksum = 0;
 
   # Build the message without a checksum.
-  my $msg = pack( ICMP_STRUCT . $heap->{data_size},
-                  ICMP_ECHO, ICMP_SUBCODE,
-                  $checksum, $pid, $seq, $heap->{data}
-                );
+  my $msg = pack(
+    ICMP_STRUCT . $heap->{data_size},
+    ICMP_ECHO, ICMP_SUBCODE, $checksum, $pid, $master_seq, $heap->{data}
+  );
 
   ### Begin checksum calculation section.
 
@@ -179,10 +191,10 @@ sub poco_ping_ping {
   ### Cease checksum calculation section.
 
   # Rebuild the message with the checksum this time.
-  $msg = pack( ICMP_STRUCT . $heap->{data_size},
-               ICMP_ECHO, ICMP_SUBCODE,
-               $checksum, $pid, $seq, $heap->{data}
-             );
+  $msg = pack(
+    ICMP_STRUCT . $heap->{data_size},
+    ICMP_ECHO, ICMP_SUBCODE, $checksum, $pid, $master_seq, $heap->{data}
+  );
 
   # Record the message's length.  This is constant, but we do it here
   # anyway.  It's also used to flag when we start requesting replies.
@@ -196,18 +208,28 @@ sub poco_ping_ping {
   }
 
   # Build an address to send the ping at.
-  my $usable_address = ( (length($address) == 4)
-                         ? $address
-                         : inet_aton($address)
-                       );
+  my $usable_address = (
+    (length($address) == 4)
+    ? $address
+    : inet_aton($address)
+  );
 
   # Return failure if an address was not resolvable.  This simulates
   # the postback behavior.
   unless (defined $usable_address) {
-    $kernel->post( $sender, $event,
-                   [ $address, $timeout, time() ],
-                   [ undef, undef, time() ],
-                 );
+    $kernel->post(
+      $sender, $event,
+      [ $address,    # REQ_ADDRESS
+        $timeout,    # REQ_TIMEOUT
+        time(),      # REQ_TIME
+        @user_args,  # REQ_USER_ARGS
+      ],
+      [ undef,   # RES_ADDRESS
+        undef,   # RES_ROUNDTRIP
+        time(),  # RES_TIME
+      ],
+    );
+    _check_for_close($kernel, $heap);
     return;
   }
 
@@ -215,25 +237,49 @@ sub poco_ping_ping {
 
   # Send the packet.  If send() fails, then we bail with an error.
   unless (send($heap->{socket_handle}, $msg, ICMP_FLAGS, $socket_address)) {
-    $kernel->post( $sender, $event,
-                   [ $address, $timeout, time() ],
-                   [ undef, undef, time() ],
-                 );
+    $kernel->post(
+      $sender, $event,
+      [ $address,    # REQ_ADDRESS
+        $timeout,    # REQ_TIMEOUT
+        time(),      # REQ_TIME
+        @user_args,  # REQ_USER_ARGS
+      ],
+      [ undef,   # RES_ADDRESS
+        undef,   # RES_ROUNDTRIP
+        time(),  # RES_TIME
+      ],
+    );
+    _check_for_close($kernel, $heap);
     return;
   }
 
   # Set a timeout based on the sequence number.
-  $kernel->delay( $seq => $timeout );
+  $kernel->delay( $master_seq => $timeout );
 
-  $heap->{ping_by_seq}->{$seq} =
-    [ # PBS_POSTBACK
-      $sender->postback($event, $address, $timeout, time(), @user_args),
-      "$sender",   # PBS_SESSION (stringified to weaken reference)
-      $address,    # PBS_ADDRESS
-      time()       # PBS_REQUEST_TIME
-    ];
+  DEBUG_PBS and warn "recording ping_by_seq($master_seq)";
+  $heap->{ping_by_seq}->{$master_seq} = [
+    # PBS_POSTBACK
+    $sender->postback(
+      $event,
+      $address,    # REQ_ADDRESS
+      $timeout,    # REQ_TIMEOUT
+      time(),      # REQ_TIME
+      @user_args,  # REQ_USER_ARGS
+    ),
+    "$sender",   # PBS_SESSION (stringified to weaken reference)
+    $address,    # PBS_ADDRESS
+    time()       # PBS_REQUEST_TIME
+  ];
 
-  $heap->{addr_to_seqr}->{$sender}->{$address} = $seq;
+  # Duplicate pings?  Forcibly time out the previous one.
+  if (exists $heap->{addr_to_seq}->{$sender}->{$address}) {
+    my $now = time();
+    my $old_seq = delete $heap->{addr_to_seq}->{$sender}->{$address};
+    my $old_info = delete $heap->{ping_by_seq}->{$old_seq};
+    $old_info->[PBS_POSTBACK]->( undef, undef, $now );
+  }
+
+  $heap->{addr_to_seq}->{$sender}->{$address} = $master_seq;
 }
 
 # Clear a ping postback by address.  The sender+address pair are a
@@ -259,9 +305,8 @@ sub poco_ping_clear {
       unless scalar(keys %{$heap->{addr_to_seq}->{$sender}});
 
     # Discard the postback for the discarded sequence number.
+    DEBUG_PBS and warn "removing ping_by_seq($seq)";
     delete $heap->{ping_by_seq}->{$seq};
-
-    # Stop the timeout, if any, for the ping.
     $kernel->delay($seq);
   }
 
@@ -269,6 +314,7 @@ sub poco_ping_clear {
   else {
     # First discard all the ping records.
     foreach my $seq (values %{$heap->{addr_to_seq}->{$sender}}) {
+      DEBUG_PBS and warn "removing ping_by_seq($seq)";
       delete $heap->{ping_by_seq}->{$seq};
       $kernel->delay($seq);
     }
@@ -277,7 +323,13 @@ sub poco_ping_clear {
     delete $heap->{addr_to_seq}->{$sender};
   }
 
-  # No more pings waiting.  Close the socket.
+  _check_for_close($kernel, $heap);
+}
+
+# XXX - NOT A POE EVENT HANDLER
+# Check to see if no more pings are waiting.  Close the socket if so.
+sub _check_for_close {
+  my ($kernel, $heap) = @_;
   unless (scalar(keys %{$heap->{ping_by_seq}}) || $heap->{keep_socket}) {
     DEBUG_SOCKET and warn "closing the raw icmp socket";
     $kernel->select_read( delete $heap->{socket_handle} );
@@ -296,6 +348,7 @@ sub poco_ping_pong {
   # Receive a message on the ICMP port.
   my $recv_message = '';
   my $from_saddr = recv($socket, $recv_message, 1500, ICMP_FLAGS);
+  return unless $from_saddr;
 
   # We haven't yet sent a message, so don't bother with whatever we've
   # received.
@@ -305,11 +358,13 @@ sub poco_ping_pong {
   my ($from_port, $from_ip) = unpack_sockaddr_in($from_saddr);
 
   # Unpack the packet itself.
-  my ( $from_type, $from_subcode,
-       $from_checksum, $from_pid, $from_seq, $from_message
-     )  = unpack( ICMP_STRUCT . $heap->{data_size},
-                  substr($recv_message, -$heap->{message_length})
-                );
+  my (
+    $from_type, $from_subcode,
+    $from_checksum, $from_pid, $from_seq, $from_message
+  )  = unpack(
+    ICMP_STRUCT . $heap->{data_size},
+    substr($recv_message, -$heap->{message_length})
+  );
 
   DEBUG and do {
     warn ",----- packet from ", inet_ntoa($from_ip), ", port $from_port\n";
@@ -327,22 +382,42 @@ sub poco_ping_pong {
   # Not from this process.  Move along.
   return unless $from_pid == $pid;
 
-  DEBUG and warn "it's from this process";
+  DEBUG and warn "it's from this process ($pid)";
 
   # Not waiting for a response with that sequence number.  Move along.
   return unless exists $heap->{ping_by_seq}->{$from_seq};
 
-  DEBUG and warn "it's one we're waiting for";
+  DEBUG and warn "it's one we're waiting for ($from_seq)";
 
   # This is the response we're looking for.  Calculate the round trip
   # time, and map it to a postback.
   my $trip_time = $now - $heap->{ping_by_seq}->{$from_seq}->[PBS_REQUEST_TIME];
-  $heap->{ping_by_seq}->{$from_seq}->[PBS_POSTBACK]->
-    ( inet_ntoa($from_ip), $trip_time, $now
+  $heap->{ping_by_seq}->{$from_seq}->[PBS_POSTBACK]->(
+    inet_ntoa($from_ip), $trip_time, $now
+  );
+
+  # It's a single-reply ping.  Clean up after it.
+  # TODO - This is a lot like the cleanup in poco_ping_default().
+  # Consider combining both if it makes sense.
+  if ($heap->{onereply}) {
+    # Delete the ping information.  Cache a copy for other cleanup.
+    DEBUG_PBS and warn "removing ping_by_seq($from_seq)";
+    my $ping_info = delete $heap->{ping_by_seq}->{$from_seq};
+    $kernel->delay($from_seq);
+
+    # Stop mapping the session+address to this sequence number.
+    delete(
+      $heap->{addr_to_seq}->{
+        $ping_info->[PBS_SESSION]
+      }->{$ping_info->[PBS_ADDRESS]}
     );
 
-  # clear the timer
-  $kernel->delay($from_seq) if $heap->{onereply};
+    # Stop tracking the session if that was the last address.
+    delete $heap->{addr_to_seq}->{$ping_info->[PBS_SESSION]}
+      unless scalar(keys %{$heap->{addr_to_seq}->{$ping_info->[PBS_SESSION]}});
+
+    _check_for_close($kernel, $heap);
+  }
 }
 
 # Default's used to catch ping timeouts, which are named after the
@@ -359,6 +434,7 @@ sub poco_ping_default {
   if (exists $heap->{ping_by_seq}->{$seq}) {
 
     # Delete the ping information, but cache a copy for other work.
+    DEBUG_PBS and warn "removing ping_by_seq($seq)";
     my $ping_info = delete $heap->{ping_by_seq}->{$seq};
 
     # Post a timer tick back to the session.  This marks the end of
@@ -366,24 +442,23 @@ sub poco_ping_default {
     $ping_info->[PBS_POSTBACK]->( undef, undef, $now );
 
     # Stop mapping the session+address to this sequence number.
-    delete( $heap->{addr_to_seq}->
-            {$ping_info->[PBS_SESSION]}->{$ping_info->[PBS_ADDRESS]}
-          );
+    delete(
+      $heap->{addr_to_seq}->{
+        $ping_info->[PBS_SESSION]
+      }->{$ping_info->[PBS_ADDRESS]}
+    );
 
     # Stop tracking the session if that was the last address.
     delete $heap->{addr_to_seq}->{$ping_info->[PBS_SESSION]}
       unless scalar(keys %{$heap->{addr_to_seq}->{$ping_info->[PBS_SESSION]}});
 
-    # Close the socket if there are no sessions waiting for responses.
-    unless (scalar(keys %{$heap->{ping_by_seq}}) || $heap->{keep_socket}) {
-      DEBUG_SOCKET and warn "closing the raw icmp socket";
-      $kernel->select_read( delete $heap->{socket_handle} );
-    }
+    _check_for_close($kernel, $heap);
 
     return 1;
   }
   else {
-    DEBUG and warn "this shouldn't technically be displayed";
+    warn "this shouldn't technically be displayed ($seq)"
+      if DEBUG and $seq =~ /^\d+$/;
 
     # Let unhandled signals pass through so we do not block SIGINT, etc.
     return 0;
@@ -396,67 +471,72 @@ __END__
 
 =head1 NAME
 
-POE::Component::Client::Ping - an ICMP ping client component
+POE::Component::Client::Ping - a non-blocking ICMP ping client
 
 =head1 SYNOPSIS
 
   use POE qw(Component::Client::Ping);
 
   POE::Component::Client::Ping->spawn(
-    Alias     => 'pingthing',  # defaults to 'pinger'
+    Alias     => "pingthing",  # defaults to "pinger"
     Timeout   => 10,           # defaults to 1 second
     OneReply  => 1             # defaults to disabled
   );
 
-  $kernel->post( 'pingthing', # Post the request to the 'pingthing'.
-                 'ping',      # Ask it to 'ping' an address.
-                 'pong',      # Have it post an answer to my 'pong' state.
-                 $address,    # This is the address we want to ping.
-                 $timeout,    # An optional timeout.  It overrides the default.
-               );
+  sub some_event_handler {
+    $kernel->post(
+      "pingthing", # Post the request to the "pingthing" component.
+      "ping",      # Ask it to "ping" an address.
+      "pong",      # Have it post an answer as a "pong" event.
+      $address,    # This is the address we want to ping.
+      $timeout,    # Optional timeout.  It overrides the default.
+    );
+  }
 
-  # This is the sub which is called when the session receives a 'pong'
+  # This is the sub which is called when the session receives a "pong"
   # event.  It handles responses from the Ping component.
   sub got_pong {
-    my ($request_packet, $response_packet) = @_[ARG0, ARG1];
+    my ($request, $response) = @_[ARG0, ARG1];
 
-    my ($req_address, $req_timeout, $req_time)      = @{$request_packet};
-    my ($resp_address, $roundtrip_time, $resp_time) = @{$response_packet};
+    my ($req_address, $req_timeout, $req_time)      = @$request;
+    my ($resp_address, $roundtrip_time, $resp_time) = @$response;
 
     # The response address is defined if this is a response.
-    # Otherwise, an undefined response address indicates that the
-    # timeout period has ended.
     if (defined $resp_address) {
-      printf( "ping to %-15.15s at %10d. pong from %-15.15s in %6.3f s\n",
-              $req_address, $request_time,
-              $resp_address, $roundtrip_time
-            );
-    }
-    else {
-      printf( "ping to %-15.15s is done.\n",
-              $req_address
-            );
+      printf(
+        "ping to %-15.15s at %10d. pong from %-15.15s in %6.3f s\n",
+        $req_address, $req_time,
+        $resp_address, $roundtrip_time,
+      );
+      return;
     }
+
+    # Otherwise the timeout period has ended.
+    printf(
+      "ping to %-15.15s is done.\n", $req_address,
+    );
   }
 
   or
 
-  use POE::Component::Client::Ping ':const';
+  use POE::Component::Client::Ping ":const";
 
-  # post an array ref as the callback to get data back to you
-  $kernel->post('pinger', 'ping', [ 'pong', @user_data ]);
+  # Post an array ref as the callback to get data back to you
+  $kernel->post("pinger", "ping", [ "pong", $user_data ]);
 
   # use the REQ_USER_ARGS constant to get to your data
   sub got_pong {
-      my ($request, $response) = @_;
-      my @req_data = @{$reqest}[REQ_USER_ARGS..$#{$request}];
+      my ($request, $response) = @_[ARG0, ARG1];
+      my $user_data = $request->[REQ_USER_ARGS];
+      ...;
   }
 
 =head1 DESCRIPTION
 
-POE::Component::Client::Ping is non-blocking ICMP ping client session.
-It lets several other sessions ping through it in parallel, and it
-lets them continue doing other things while they wait for responses.
+POE::Component::Client::Ping is non-blocking ICMP ping client.  It
+lets several other sessions ping through it in parallel, and it lets
+them continue doing other things while they wait for responses.
+
 Ping client components are not proper objects.  Instead of being
 created, as most objects are, they are "spawned" as separate sessions.
 To avoid confusion (and hopefully not cause other confusion), they
@@ -469,102 +549,123 @@ PoCo::Client::Ping's C<spawn> method takes a few named parameters:
 
 =item Alias => $session_alias
 
-C<Alias> sets the name by which the session will be known.  If no
-alias is given, the component defaults to "pinger".  The alias lets
-several sessions interact with resolver components without keeping (or
-even knowing) hard references to them.  It is possible to spawn
-several Ping components with different names, but the author can find
-no point in doing this.
+C<Alias> sets the component's alias.  It is the target of post()
+calls.  See the synopsis.  The alias defaults to "pinger".
 
 =item Socket => $raw_socket
 
-C<Socket> allows for the caller to open an existing raw socket rather
-than letting PoCo::Client::Ping try and open one itself.  This can be
-used to let startup code create the socket while running with enhanced
-privileges, which are dropped.  People who don't want to audit POE for
-security holes will find this useful since they won't need to run POE
-as root.
+C<Socket> allows developers to open an existing raw socket rather
+than letting the component attempt opening one itself.  If omitted,
+the component will create its own raw socket.
+
+This is useful for people who would rather not perform a security
+audit on POE, since it allows them to create a raw socket in their own
+code and then run POE at reduced privileges.
 
 =item Timeout => $ping_timeout
 
-C<Timeout> specifies the default amount of time a Ping component will
-wait for an ICMP echo reply.  C<$ping_timeout> contains a real number
-indicating how many seconds to wait.  It's possible and meaningful to
-set the timeout to a fractional number of seconds.
+C<Timeout> sets the default amount of time a Ping component will wait
+for an ICMP echo reply, in seconds.  It is 1 by default.  It's
+possible and meaningful to set the timeout to a fractional number of
+seconds.
 
-The default timeout is only used for ping requests that don't include
+This default timeout is only used for ping requests that don't include
 their own timeouts.
 
 =item OneReply => 0|1
 
-C<OneReply>, if set, tells the Ping component to clear the timeout upon
-successful ping.  The Ping component will thus only post one reply back
-to the calling module, upon either success or timeout, but not both.
-This is useful if you are checking if one host is reachable or not as
-you will only receive one reply.  By default, this setting is disabled,
-meaning that you will get the ping timeout in addition to any ping 
-replies.  This is useful if you are expecting multiple replies from one
-ping, such as when pinging a subnet.
- 
+Set C<OneReply> to prevent the Ping component from waiting the full
+timeout period for replies.  Normally the ICMP protocol allows for
+multiple replies to a single request, so it's proper to wait for late
+responses.  This option disables the wait, ending the ping transaction
+at the first response.  Any subsequent responses will be silently
+ignored.
+
+C<OneReply> is disabled by default, and a single successful request
+will generate at least two responses.  The first response is a
+successful ICMP ECHO REPLY event.  The second is an undefined response
+event, signifying that the timeout period has ended.
+
+A ping request will generate exactly one reply when C<OneReply> is
+enabled.  This reply will represent either the first ICMP ECHO REPLY
+to arrive or that the timeout period has ended.
+
 =back
 
-Sessions communicate asynchronously with PoCo::Client::Ping.  They
-post ping requests to it, and it posts responses back.
+Sessions communicate asynchronously with the Client::Ping component.
+They post ping requests to it, and they receive pong events back.
 
-Requests are posted to the component's "ping" state.  They include the
-name of a state to post responses back to, an address to ping, and an
-optional amount of time to wait for responses.  The address may be a
-numeric dotted quad, a packed inet_aton address, or a host name.  Host
-names aren't recommended: they're looked up for every ping request,
-and that's slow.  The optional timeout overrides the one set when
-C<spawn> is called.
+Requests are posted to the component's "ping" handler.  They include
+the name of an event to post back, an address to ping, and an optional
+amount of time to wait for responses.  The address may be a numeric
+dotted quad, a packed inet_aton address, or a host name.  Host names
+are not recommended: they must be looked up for every ping request,
+and DNS lookups can be very slow.  The optional timeout overrides the
+one set when C<spawn> is called.
 
-Ping responses come with two list references:
+Ping responses come with two array references:
 
-  my ($request_packet, $response_packet) = @_[ARG0, ARG1];
+  my ($request, $response) = @_[ARG0, ARG1];
 
-C<$request_packet> contains information about the original request:
+C<$request> contains information about the original request:
 
-  my ($request_address, $request_timeout, $request_time) = @{$request_packet};
+  my (
+    $req_address, $req_timeout, $req_time, $req_user_args,
+  ) = @$request;
 
 =over 2
 
-=item C<$request_address>
+=item C<$req_address>
 
 This is the original request address.  It matches the address posted
-along with the original 'ping' request.
+along with the original "ping" request.
 
-=item C<$request_timeout>
+It is useful along with C<$req_user_args> for pairing requests with
+their corresponding responses.
+
+=item C<$req_timeout>
 
 This is the original request timeout.  It's either the one passed with
-the 'ping' request or the default timeout set with C<spawn>.
+the "ping" request or the default timeout set with C<spawn>.
 
-=item C<$request_time>
+=item C<$req_time>
 
-This is the time that the 'ping' event was received by the Ping
+This is the time that the "ping" event was received by the Ping
 component.  It is a real number based on the current system's time()
 epoch.
 
+=item C<$req_user_args>
+
+This is a scalar containing arbitrary data that can be sent along with
+a request.  It's often used to provide continuity between requests and
+their responses.  C<$req_user_args> may contain a reference to some
+larger data structure.
+
+To use it, replace the response event with an array reference in the
+original request.  The array reference should contain two items: the
+actual response event and a scalar with the context data the program
+needs back.  See the SYNOPSIS for an example.
+
 =back
 
-C<$response_packet> contains information about the ICMP ping response.
-There may be multiple responses for a single ping.
+C<$response> contains information about the ICMP ping response.  There
+may be multiple responses for a single request.
 
-  my ($response_address, $roundtrip_time, $reply_time) = @{$response_packet};
+  my ($response_address, $roundtrip_time, $reply_time) = @$response;
 
 =over 2
 
 =item C<$response_address>
 
 This is the address that responded to the ICMP echo request.  It may
-be different than C<$request_address>, especially if the request
-address was a subnet.
+be different than C<$request_address>, especially if the request was
+sent to a broadcast address.
 
 C<$response_address> will be undefined if C<$request_timeout> seconds
 have elapsed.  This marks the end of responses for a given request.
-Programs can assume that a request address will not send more
-responses after this, and they may use the end-of-responses marker to
-initiate another ping request.
+Programs can assume that no more responses will be sent for the
+request address.  They may use this marker to initiate another ping
+request.
 
 =item C<$roundtrip_time>
 
@@ -579,21 +680,21 @@ real number based on the current system's time() epoch.
 
 =back
 
-If the :const tagset is imported the following constants will be exported:
+If the ":const" tagset is imported the following constants will be
+exported:
 
-REQ_ADDRESS, REQ_TIMEOUT, REQ_TIME, REQ_USER_ARGS, RES_ADDRESS, 
-RES_ROUNDTRIP, RES_TIME
+REQ_ADDRESS, REQ_TIMEOUT, REQ_TIME
+REQ_USER_ARGS, RES_ADDRESS, RES_ROUNDTRIP, RES_TIME
 
 =head1 SEE ALSO
 
 This component's ICMP ping code was lifted from Net::Ping, which is an
-excellent module times when you don't mind pinging single hosts and
-only knowing whether they're alive.
+excellent module when you only need to ping one host at a time.
 
 See POE, of course, which includes a lot of documentation about how
-POE does things.
+POE works.
 
-Also see the test program, t/01_ping.t, in the PoCo::Client::Ping
+Also see the test program, t/01_ping.t, in the component's
 distribution.
 
 =head1 BUGS
@@ -602,11 +703,13 @@ None currently known.
 
 =head1 AUTHOR & COPYRIGHTS
 
-POE::Component::Client::Ping is Copyright 1999-2002 by Rocco Caputo.
+POE::Component::Client::Ping is Copyright 1999-2004 by Rocco Caputo.
 All rights are reserved.  POE::Component::Client::Ping is free
 software; you may redistribute it and/or modify it under the same
 terms as Perl itself.
 
-Rocco may be contacted by e-mail via rcaputo@cpan.org.
+Rocco may be contacted by e-mail via <rcaputo@cpan.org>.
+
+You can learn more about POE at <http://poe.perl.org/>.
 
 =cut
